import { Application, Router } from 'https://deno.land/x/oak/mod.ts';
import { Client } from "https://deno.land/x/postgres@v0.17.0/mod.ts";

const app = new Application();
const router = new Router();

//---------dotenv info-------------
import { load } from "https://deno.land/std/dotenv/mod.ts";

const env = await load();
const hostname = env["HOSTNAME"];
const port     = env["PORT"];
const user     = env["USER"];
const password = env["PASSWORD"];
const database = env["DATABASE"];
const app_port = env["APP_PORT"];


// Configure database connection
const client = new Client({
  hostname:hostname 
  ,port:    port     
  ,user:    user     
  ,password:password 
  ,database:database 
  ,applicationName: "pricing guidance"
});

await client.connect();

// Load SQL from file
const query = await Deno.readTextFile("sql/get.pg.sql");

function apply_guidance(doc: any) {
  let mostRelevantMarketPrice = null;
  let mostRelevantMarketKey = null;
  let mostRelevantMarketSeason = null;
  let highestMarketRelevanceLevel = -1;

  let mostRelevantCustomerPrice = null;
  let mostRelevantCustomerKey = null;
  let mostRelevantCustomerSeason = null;
  let highestCustomerRelevanceLevel = -1;

  // Function to update price and assign relevance indicator
  function setAnchors(items, channelFirstChar, v1ds, v0ds, histKey) {
    for (let item of items) {
      // Update the last_price with the most recent price
      const years = Object.keys(item.season);
      if (years.length > 0) {
        const recentYear = Math.max(...years.map(Number));
        const lastPrice = item.season[recentYear].price_usd;
        item.last_price = lastPrice;
        item.last_season = recentYear;
      } else {
        item.last_price = null; // or some default value as appropriate
      }

      // Initialize relevance as numeric value
      let marketRelevance = 0; // Assume 0 is 'not relevant'
      let customerRelevance = 0; // Assume 0 is 'not relevant'

      // Check if the first character of the item's channel matches the first character of the document's channel
      if (item.chan.charAt(0) === channelFirstChar) {
        marketRelevance = 1; // 'relevant'

        // Further refine relevance based on v1ds and v0ds
        if (item.v1ds === v1ds) {
          marketRelevance = 2; // 'most relevant' because v1ds matches
          // Check for customer relevance if 'cust' key exists
          customerRelevance = item.cust ? 2 : 1
        } else if (item.v0ds === v0ds) {
          marketRelevance = marketRelevance === 2 ? 2 : 1; // Keep relevance as is if v1ds was matched, otherwise it's just 'relevant'
        }
      }


      // Assign the calculated relevance to the item
      item.marketRelevance = marketRelevance;
      item.customerRelevance = customerRelevance;

      // Update the most relevant market price if this item's relevance is higher and it doesn't have a 'cust' key
      if (marketRelevance > highestMarketRelevanceLevel) {
        highestMarketRelevanceLevel = marketRelevance;
        mostRelevantMarketPrice = item.last_price;
        mostRelevantMarketKey = histKey;
        mostRelevantMarketSeason = item.last_season; // Assuming 'season' is the key where the season info is stored
      }

      // Update the most relevant customer price if this item's relevance is higher and it has a 'cust' key
      if (customerRelevance > highestCustomerRelevanceLevel) {
        highestCustomerRelevanceLevel = customerRelevance;
        mostRelevantCustomerPrice = item.last_price;
        mostRelevantCustomerKey = histKey;
        mostRelevantCustomerSeason = item.last_season; // Assuming 'season' is the key where the season info is stored
      }
    }
  }

  // Iterate over each key in the "hist" object
  for (let key of Object.keys(doc.hist)) {
    // Update price and relevance for each item in the current key
    setAnchors(doc.hist[key], doc.chan[0], doc.v1ds, doc.v0ds, key);
  }

  // Assign the most relevant market price and key to the top level of the document
  if (mostRelevantMarketPrice !== null) {
    doc.mostRelevantMarketPriceInfo = {
      price: mostRelevantMarketPrice,
      histKey: mostRelevantMarketKey,
      season: mostRelevantMarketSeason
    };
  }

  // Assign the most relevant customer price and key to the top level of the document
  if (mostRelevantCustomerPrice !== null) {
    doc.mostRelevantCustomerPriceInfo = {
      price: mostRelevantCustomerPrice,
      histKey: mostRelevantCustomerKey,
      season: mostRelevantCustomerSeason
    };
  }

  doc.targetPrice = doc.v1tp ?? doc.v0tp ?? null;
  // Determine the anchor price and source
  if (doc.targetPrice !== undefined && (mostRelevantCustomerPrice === undefined || doc.targetPrice < mostRelevantCustomerPrice)) {
    doc.anchorPrice = doc.targetPrice;
    doc.anchorSource = 'targetPrice';
  } else if (mostRelevantCustomerPrice !== undefined) {
    doc.anchorPrice = mostRelevantCustomerPrice;
    doc.anchorSource = 'customerPrice';
  } else {
    doc.anchorPrice = null;
    doc.anchorSource = 'none'; // or any other default value you wish to indicate no anchor price was found
  }

  return doc;
}


// Function to update each item with the most recent price
function updateWithMostRecentPrice(items) {
  for (let item of items) {
    // Find the most recent price
    //const lastPrice = findMostRecentPrice(item.season);
    const years = Object.keys(item.season);
    const recentYear = Math.max(...years);
    const lastPrice = item.season[recentYear].price_usd;
    // Append the last_price to the item
    item.last_price = lastPrice;
    item.last_year = recentYear;
  }
}

// Define a route to retrieve values from the database using parameters
router.get('/code_price/:billcode/:shipcode/:partcode/:qty', async (ctx) => {

    const partcode = ctx.params.partcode;
    const billcode = ctx.params.billcode;
    const shipcode = ctx.params.shipcode;
    const qty = ctx.params.qty;

    //console.log(partcode)
    //console.log(customer)
    const result = await client.queryObject({args: [billcode, shipcode, partcode, qty], text: query} );
    const procd = apply_guidance(result.rows[0]["doc"])
    ctx.response.body = procd
});

app.use(router.routes());
app.use(router.allowedMethods());

// Start the server
console.log('Server is running on http://usmidsap02:8090');
await app.listen({ port: 8090 });

